#define sock_implementation
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdarg.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <err.h>
#include <errno.h>
#include <netinet/in.h>
#include "sock.h"


int sock_sendall(int sock, char *m, int len)
{
  while(len) 
  {
	int r= send(sock, m, len,MSG_NOSIGNAL);
	if (r<=0) {
		if (errno==EINTR)
			continue;
		warn("send");
		break;
	}
	len-= r;
	m+= r;
  }
  return len;
}

int sock_recvall(int sock, char *m, int len)
{
  while(len) 
  {
	int r= recv(sock, m, len,0);
	if (r<=0) {
		if (errno==EINTR)
			continue;
		warn("recv");
		break;
	}
	len-= r;
	m+= r;
  }
  return len;
}

int sock_create_server(int port)
{
	int s;
	int ec;
	char *f;
	s= socket(PF_INET, SOCK_STREAM, 0);
	if (s<0) {
		ec= errno;
		f= "socket";
		goto fail_socket;
	}
	if (port!=0) {
		int yes= 1;
		struct sockaddr_in addr;
		setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(yes));
		memset(&addr, 0, sizeof(addr));
		addr.sin_family= AF_INET;
		addr.sin_port= htons( (short) port);
		addr.sin_addr.s_addr= INADDR_ANY;
		if (bind(s, (struct sockaddr*) &addr,sizeof(addr))) {
			ec= errno;
			f= "bind";
			goto fail_bind;
		}
	}
	if (listen(s, 5)) {
		ec= errno;
		f= "listen";
		goto fail_listen;
	}
	
	return s;
fail_listen:
fail_bind:
	shutdown(s,3);
	close(s);
fail_socket:
	fprintf(stderr,"%s: %s\n",f, strerror(ec));
	exit(1);
	return -1;
}


int sock_get_port(int s)
{
	struct sockaddr_in addr;
	socklen_t len= sizeof(addr);
	if (getsockname(s, (struct sockaddr*) &addr,&len)) 
        {
		fprintf(stderr,"getsockname: %s\n",strerror(errno));
		exit(1);
		return -1;
	}
	return ntohs(addr.sin_port);
}


int sock_connect(char *host,int port)
{
	int s;
	s= socket(PF_INET, SOCK_STREAM, 0);
	if (s<0)
		return -1;
	{
		struct sockaddr_in addr;
		int len= sizeof(addr);
		memset(&addr, 0, sizeof(addr));
		addr.sin_family= AF_INET;
		addr.sin_port= htons( (short) port);
		inet_aton(host,&addr.sin_addr);
		if (connect(s,(struct sockaddr*) &addr, len))
			return -1;
	}
	return s;
}

#if 0

int xrecv(int s,char *z,int len,int opt) {
	int r;
	int ec;
	r= recv(s,z,len,opt | MSG_NOSIGNAL);
	ec= errno;
	printf("recv(port %d) returned %d: ",sock_get_port(s),r);
	if (r>0) { 
		printf("\n'");
		fwrite(z,1,r,stdout);
		printf("'\n");
	} else printf("errno= %d\n",ec);
	errno= ec;
	return r;
}

int xsend(int s,char *z,int len,int opt) {
	int r;
	int ec;
	r= send(s,z,len,opt | MSG_NOSIGNAL);
	ec= errno;
	printf("send(port %d) returned %d: ",sock_get_port(s),r);
	if (r>0) { 
		printf("\n'");
		fwrite(z,1,r,stdout);
		printf("'\n");
	} else printf("errno= %d\n",ec);
	errno= ec;
	return r;
}
#endif

void sock_close(int sock)
{
  shutdown(sock,3);
  close(sock);
}
